import { RPC, RequestOf, ResponseOf, RPCs } from '@previewjs/api';
import { TypeAnalyzer } from '@previewjs/type-analyzer';
import { Reader } from '@previewjs/vfs';
import { Logger } from 'pino';
import { ComponentAnalyzer } from '@previewjs/component-analyzer-api';
import vite from 'vite';
import * as express from 'express';

type PackageDependencies = Record<string, {
    /** @deprecated */
    majorVersion: number;
    readInstalledVersion(): Promise<string | null>;
}>;

interface FrameworkPluginFactory {
    isCompatible(dependencies: PackageDependencies): Promise<boolean>;
    create(options: {
        rootDir: string;
        reader: Reader;
        logger: Logger;
        dependencies: PackageDependencies;
    }): Promise<FrameworkPlugin>;
}
interface FrameworkPlugin extends ComponentAnalyzer {
    readonly pluginApiVersion?: number;
    readonly name: string;
    readonly defaultWrapperPath: string;
    readonly previewDirPath: string;
    readonly viteConfig: (configuredPlugins: vite.Plugin[]) => vite.UserConfig;
    dispose(): void;
}

type RegisterRPC = <E extends RPC<any, any>>(endpoint: E, handler: RequestHandler<RequestOf<E>, ResponseOf<E>>) => void;
type RequestHandler<Req, Res> = (request: Req) => Promise<Res>;

type SetupPreviewEnvironment = (options: {
    versionCode?: string;
    registerRPC: RegisterRPC;
    workspace: Workspace;
}) => Promise<{
    middlewares?: express.RequestHandler[];
}>;

declare function setupFrameworkPlugin({ rootDir, frameworkPlugins, reader, logger, }: {
    rootDir: string;
    frameworkPlugins: FrameworkPluginFactory[];
    reader: Reader;
    logger: Logger;
}): Promise<FrameworkPlugin | null>;

declare function createWorkspace({ rootDir, reader, frameworkPlugin, logger, setupEnvironment, }: {
    rootDir: string;
    frameworkPlugin: FrameworkPlugin;
    logger: Logger;
    reader: Reader;
    setupEnvironment?: SetupPreviewEnvironment;
}): Promise<Workspace>;
/**
 * Returns the absolute directory path of the closest ancestor containing package.json.
 */
declare function findWorkspaceRoot(absoluteFilePath: string): string | null;
interface Workspace {
    rootDir: string;
    reader: Reader;
    typeAnalyzer: Omit<TypeAnalyzer, "dispose">;
    detectComponents(options?: RequestOf<typeof RPCs.DetectComponents>): Promise<ResponseOf<typeof RPCs.DetectComponents>>;
    computeProps(options: RequestOf<typeof RPCs.ComputeProps>): Promise<ResponseOf<typeof RPCs.ComputeProps>>;
    preview: {
        start(allocatePort?: () => Promise<number>): Promise<Preview>;
    };
    dispose(): Promise<void>;
}
interface Preview {
    url(): string;
    stop(): Promise<void>;
}

export { FrameworkPlugin, FrameworkPluginFactory, PackageDependencies, Preview, SetupPreviewEnvironment, Workspace, createWorkspace, findWorkspaceRoot, setupFrameworkPlugin };
